//=============================================================================
// Picaso SGC implementation for Touch screen
//=============================================================================
#include "globals.h"
#include "OLEDSGC.h"

#include <SPI.h>
#include <Wire.h>      // this is needed even tho we aren't using it
#include <Adafruit_ILI9341.h>
#include <Adafruit_STMPE610.h>


// This is calibration data for the raw touch data to the screen coordinates
#define TS_MINX 150
#define TS_MINY 130
#define TS_MAXX 3800
#define TS_MAXY 4000

// The STMPE610 uses hardware SPI on the shield, and #8
#define STMPE_CS 8
Adafruit_STMPE610 ts = Adafruit_STMPE610(STMPE_CS);

// The display also uses hardware SPI, plus #9 & #10
#define TFT_CS 10
#define TFT_DC 9
Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);


// Define an instance of this class.


OLEDSGC g_plcd;

//=============================================================================
// InitOLED: Initialize our OLED display
//=============================================================================
void  OLEDSGC::Init(byte bResetPin) {                // lets initialize our display
  byte bBuffer[10];                                    // Have a default buffer

  tft.begin();
  tft.setRotation(3);
  if (!ts.begin()) {
    Serial.println("Couldn't start touchscreen controller");
    while (1);
  }
  Serial.println("Touchscreen started");
  
  tft.fillScreen(ILI9341_BLACK);

}  


//=============================================================================
// OLEDDisplayString
//=============================================================================
void OLEDSGC::DisplayString(byte x, byte y, byte bFont, word wColor, const char *pb) {
  DisplayGraphicString(x*5, y*7, bFont, wColor, pb);

}

//=============================================================================
// OLEDDisplayGraphicString
//=============================================================================
void OLEDSGC::DisplayGraphicString(word x, word y, byte bFont, word wColor, const char *pb) {


  tft.setCursor(x, y);
  
  if (_ulFontBackgroundColor != -1)
    tft.setTextColor(wColor, _ulFontBackgroundColor);
  else
    tft.setTextColor(wColor);
  
  tft.print(pb); 
}


//=============================================================================
// OLEDDisplayNum
//=============================================================================
void OLEDSGC::DisplayNum(byte x, byte y, byte bFont, word wColor, word w, byte bCnt) {
  char ab[6];        // can hold the max string a word can 

  // convert number to string, that is right justified in our string.
  ab[bCnt] = 0;        // last character
  while (bCnt > 0) {
    bCnt--;
    ab[bCnt] = '0' + w % 10;  // get the last digit probably faster ways
    w /= 10;            // Setup for the next digit
  }
  DisplayGraphicString(x*5, y*7, bFont, wColor, ab);
}


//=============================================================================
// ClearScreen: clears the screen
//=============================================================================
void OLEDSGC::ClearScreen(void) {
  tft.fillScreen(ILI9341_BLACK);
}

//=============================================================================
// DrawRect: Draw a rectangle
//=============================================================================
void OLEDSGC::DrawRect(word x1, word y1, word x2, word y2, word wColor) {
  if (_fSolid)
    tft.fillRect(x1, y1, x2-x1+1, y2-y1+1, wColor);
  else
    tft.drawRect(x1, y1, x2-x1+1, y2-y1+1, wColor);
}


//=============================================================================
// ScreenCopyPaste: Copy screen bits from Source to Dest
//=============================================================================
void OLEDSGC::ScreenCopyPaste(word xS, word yS, word xD, word yD, word dx, word dy) {
#ifdef LATER
  WaitForPreviousCommandToComplete(100);
  OLEDSerial.print("c");
  OLEDSerial.write(xS>>8);
  OLEDSerial.write(xS & 0xff);
  OLEDSerial.write(yS>>8);
  OLEDSerial.write(yS & 0xff);
  OLEDSerial.write(xD>>8);
  OLEDSerial.write(xD & 0xff);
  OLEDSerial.write(yD>>8);
  OLEDSerial.write(yD & 0xff);
  OLEDSerial.write(dx>>8);
  OLEDSerial.write(dx & 0xff);
  OLEDSerial.write(dy>>8);
  OLEDSerial.write(dy & 0xff);

  SetCommandActive();
#endif  
}

//=============================================================================
// GetTouchStatus - Get the touch status assocated with our display
//=============================================================================
boolean OLEDSGC::FTouchEventsAvail()
{
  return !ts.bufferEmpty();
}

//=============================================================================
// GetTouchStatus - Get the touch status assocated with our display
//=============================================================================
boolean OLEDSGC::GetTouchPoint(word *pwX, word *pwY)
{
   TS_Point p = ts.getPoint();

  // Scale from ~0->4000 to tft.width using the calibration #'s
  *pwX = map(p.y, TS_MINX, TS_MAXX, tft.width(), 0);
  *pwY = map(p.x, TS_MINY, TS_MAXY, 0, tft.height());
 
}
//=============================================================================
// GetTouchStatus - Get the touch status assocated with our display
//=============================================================================
boolean OLEDSGC::FTouched() 
{
  return ts.touched();
}

//=============================================================================
// GetTouchStatus - Get the touch status assocated with our display
//=============================================================================
 #ifdef later 
boolean OLEDSGC::GetTouchStatus(byte bMode, word *wX, word *wY)    // get the two words of data associated with the touch panel
{
  byte ab[4];
  boolean fRet;

  // Make sure we don't have any pending commands that we are waiting on...
  WaitForPreviousCommandToComplete(100);

  // Now we need to output the request.
  OLEDSerial.write("o");
  OLEDSerial.write(bMode);

  // Now we should receive 4 bytes of status back.
  fRet = WaitforResp(ab, 4, 100);

  *wX = (ab[0] << 8) | ab[1];

  // The Y is optional as we may not care if we are simply getting status.
  if (wY)
    *wY = (ab[2] << 8) | ab[3];

  return fRet;
}
#endif  



//#############################################################################################################

//=============================================================================================================
//  Display Object Implementation.
//=============================================================================================================
inline void  DISPOBJ::Enable(boolean fEnabled) {
  _fEnabled = fEnabled;
}


//=============================================================================================================
//  Display Object: Button
//=============================================================================================================
void BUTTON::Init(word x, word y, word dx, word dy, word wBtnClr, word wHiClr, word wTextClr, char *psz, word wVal)  {
  // Save away all of the data...
  // first the base objects stuff...
  _x = x;
  _w =dx;
  _y = y;
  _h = dy;
  _fEnabled = true;        // default to enabled...

  _wBtnClr = wBtnClr;
  _wHiClr = wHiClr;
  _wTextClr = _wTextClr;
  _psz = psz;
  _wVal = wVal;
  _fPressed = false;
}


void BUTTON::Draw() {

  // BUGBUG:: Hard coded font size... 10x16
  word wXText = _x + _w/2 - strlen(_psz)*10/2;  // Could precalc and save also verify that text fits...
  word wYText = _y + _h/2 - 16/2;
#if 0
  DBGSerial.print("Btn: ");
  DBGSerial.write(_psz);
  DBGSerial.write(" ");
  DBGSerial.print(_x, DEC);
  DBGSerial.write(" ");
  DBGSerial.print(_y, DEC);
  DBGSerial.write(" ");
  DBGSerial.print(_w, DEC);
  DBGSerial.write(" ");
  DBGSerial.print(_h, DEC);
  DBGSerial.write(" ");
  DBGSerial.print(wXText, DEC);
  DBGSerial.write(" ");
  DBGSerial.println(wYText, DEC);
#endif
  g_plcd.SetFontBackgroundColor(-1);    // Opaque

  if (_fPressed) {
    tft.fillRect(_x, _y, _w, _h, _wHiClr);
    tft.drawRect(_x+2, _y+2, _w-4, _h-4, _wBtnClr);

    // Now output graphic text...
    g_plcd.DisplayGraphicString(wXText, wYText , 3, _wTextClr, _psz);

  } 
  else {
    g_plcd.SetSolidPin(true);
    tft.fillRect(_x, _y, _w, _h, _wBtnClr);
    g_plcd.SetSolidPin(false);
    tft.drawRect(_x+2, _y+2, _w-4, _h-4, _wHiClr);

    // Now output graphic text...
    g_plcd.DisplayGraphicString(wXText, wYText , 3, _wTextClr, _psz);
  }
}

// Assumes that we got a button down event, we pass in the X, Y from this event and it will check to see if the coordinates
// are within the object and process it. If the up happens in the same object then we will return the object ID, else we will
// 
word BUTTON::ProcessTouch(word x, word y) {

  word wTouchState;

  if (!((x >= _x) && (x <= (_x+_w)) && (y >= _y) && (y <= (_y+_h))))
    return 0x0;        // Special value that says Not within the object...

  SetPressed(true);
  Draw();    // Show it as depressed.

  // Now lets wait for the touch to release...
  while (g_plcd.FTouched()) {
    g_plcd.GetTouchPoint(&x, &y);
  } 

  SetPressed(false);
  Draw();

  // And see if we were still inside the button    
  if ((x >= _x) && (x <= (_x+_w)) && (y >= _y) && (y <= (_y+_h)))
    return _wVal;

  return 0xffff;    // not on the button any more...
}
//=============================================================================================================
//  Display Object: slider
//=============================================================================================================
void SLIDER::Init (word x, word y, word dx, word dy, word wClr, word wHiClr, word wBackClr, word wMin, word wMax, word wVal) {
  // first the base objects stuff...
  _x = x;
  _w = dx;
  _y = y;
  _h = dy;
  _fEnabled = true;        // default to enabled...
  _fVert = (dy > dx);

  // Now the slider specific fields
  _wClr = wClr;
  _wHiClr = wHiClr;
  _wBackClr = _wBackClr;
  _fPressed = false;

  _wMin = wMin;
  _wMax = wMax;
  _wVal = wVal;
  MapValToDispVal();
}

void SLIDER::MapValToDispVal(void) {
  if (_fVert)
    // In vertical we start to fill from bottom to top... 
    _wDispVal = map(_wVal, _wMin, _wMax, _y+_h, _y+1);
  else
    _wDispVal = map(_wVal, _wMin, _wMax, _x+1, _x+_w);
}    

word SLIDER::MapDispValToVal(word wXorY) {
  if (_fVert)
    return map(wXorY, _y+1, _y+_h, _wMax, _wMin);

  return map(wXorY, _x+1, _x+_w, _wMin, _wMax);
}    


void SLIDER::Draw(void) {
  // Start off real simple simple filled rect to where the current value is... May also have simple text value...
  //First lets draw the outline frame around our object...
  tft.drawRect(_x, _y, _w, _h, _wClr);
  if (_fVert) {
    // In vertical we start to fill from bottom to top... 
    if ( _wDispVal > (_y+1)) 
      tft.fillRect(_x+1, _y+1, _w-2,  _wDispVal-_y, _wBackClr);
    if ( _wDispVal < (_y+_h-1))    
      tft.fillRect(_x+1,  _wDispVal, _w-2, _h - (_wDispVal-_y), _wHiClr);
  } 
  else {
    if ( _wDispVal > (_x+1)) 
      tft.fillRect(_x+1, _y+1,  _wDispVal-_x, _h-2, _wHiClr);
    if ( _wDispVal < (_x+_w-1))    
      tft.fillRect( _wDispVal+1, _y+1, _w - (_wDispVal-_x), _h-2, _wBackClr);
  }
}

word SLIDER::ProcessTouch(word x, word y) {
  word wTouchState;

  if (!((x >= _x) && (x <= (_x+_w)) && (y >= _y) && (y <= (_y+_h))))
    return 0x0;        // Special value that says Not within the object...

  SetPressed(true);
  Draw();    // Show it as depressed.

  // Now lets wait for the touch to release...
  while (g_plcd.FTouched()) {
    g_plcd.GetTouchPoint(&x, &y);
    if (_fVert) {
      if ((y >= _y) && (y <= (_y+_h))) {
        _wDispVal = y;
        Draw();    // try to redraw....
      }
    } 
    else {
      if ((x >= _x) && (x <= (_x+_w))) {
        _wDispVal = x;
        Draw();    // try to redraw....
      }
    }
  } 
  while (g_plcd.FTouchEventsAvail()) {
    g_plcd.GetTouchPoint(&x, &y);
  }

  SetPressed(false);

  // And see if we were still inside the button    
  if ((x >= _x) && (x < (_x+_w)) && (y >= _y) && (y < (_y+_h))) {
    if (_fVert)
      SetValue(MapDispValToVal(y));
    else
      SetValue(MapDispValToVal(x));
    return 1;
  }

  // Otherwise make sure we are mapped back to the correct value...
  MapValToDispVal();
  Draw();
  return 0xffff;    // not on the button any more...
}


void SLIDER::SetValue(word wVal) {
  if (wVal != _wVal) {
    _wVal = wVal;

    MapValToDispVal();

    Draw();    // redraw the slider
  }
}








